
// TODO: Make far ptr relative to root

`include "raycast_defines.v"

module raycast_master
    (

        // WB Master
       m_wb_adr_o, m_wb_sel_o, m_wb_we_o,
       m_wb_dat_o, m_wb_dat_i, m_wb_cyc_o,
       m_wb_stb_o, m_wb_ack_i, m_wb_err_i,
       m_wb_cti_o, m_wb_bte_o,

        // Controller i/o
        ctrl_wb_adr_i, ctrl_wb_sel_i, ctrl_wb_we_i,
        ctrl_wb_dat_o, ctrl_wb_dat_i, ctrl_wb_cyc_i,
        ctrl_wb_stb_i, ctrl_wb_ack_o,
        ctrl_wb_cti_i, ctrl_wb_bte_i,

        // Core i/o
`ifdef CORE0
        c0_wb_adr_i, c0_wb_dat_o, c0_wb_cyc_i, c0_wb_stb_i,
        c0_wb_ack_o,
`endif
`ifdef CORE1
        c1_wb_adr_i, c1_wb_dat_o, c1_wb_cyc_i, c1_wb_stb_i,
        c1_wb_ack_o,
`endif
`ifdef CORE2
        c2_wb_adr_i, c2_wb_dat_o, c2_wb_cyc_i, c2_wb_stb_i,
        c2_wb_ack_o,
`endif
`ifdef CORE3
        c3_wb_adr_i, c3_wb_dat_o, c3_wb_cyc_i, c3_wb_stb_i,
        c3_wb_ack_o,
`endif

        wb_clk, wb_rst,

    );

    // = Parameters =
    parameter S_IDLE = 0, S_CTRL = 1, S_CORE0 = 2, S_CORE1 = 3, S_CORE2 = 4, S_CORE3 = 5;
    // --

    // = Ports =
    input wb_clk;
    input wb_rst;

    // WISHBONE master
   output reg [31:0]        m_wb_adr_o;
   output reg [3:0]         m_wb_sel_o;
   output reg           m_wb_we_o;
   input [31:0]         m_wb_dat_i;
   output reg [31:0]        m_wb_dat_o;
   output reg               m_wb_cyc_o;
   output reg               m_wb_stb_o;
   input                m_wb_ack_i;
   input                m_wb_err_i;
   output reg [2:0]         m_wb_cti_o;   // Cycle Type Identifier
   output reg [1:0]         m_wb_bte_o;   // Burst Type Extension


    input [31:0]        ctrl_wb_adr_i;
   input [3:0]      ctrl_wb_sel_i;
   input                ctrl_wb_we_i;
   output reg [31:0]    ctrl_wb_dat_o;
   input [31:0]     ctrl_wb_dat_i;
   input            ctrl_wb_cyc_i;
   input            ctrl_wb_stb_i;
   output reg           ctrl_wb_ack_o;
    input [2:0] ctrl_wb_cti_i;
    input [1:0] ctrl_wb_bte_i;

`ifdef CORE0
   input [31:0]     c0_wb_adr_i;
   output reg [31:0] c0_wb_dat_o;
   input            c0_wb_cyc_i;
   input            c0_wb_stb_i;
   output reg        c0_wb_ack_o;
`endif
`ifdef CORE1
   input [31:0]     c1_wb_adr_i;
   output reg [31:0] c1_wb_dat_o;
   input            c1_wb_cyc_i;
   input            c1_wb_stb_i;
   output reg        c1_wb_ack_o;
`endif
`ifdef CORE2
   input [31:0]     c2_wb_adr_i;
   output reg [31:0] c2_wb_dat_o;
   input            c2_wb_cyc_i;
   input            c2_wb_stb_i;
   output reg        c2_wb_ack_o;
`endif
`ifdef CORE3
   input [31:0]     c3_wb_adr_i;
   output reg [31:0] c3_wb_dat_o;
   input            c3_wb_cyc_i;
   input            c3_wb_stb_i;
   output reg        c3_wb_ack_o;
`endif
    // --

    // == Registers/memories =
    reg [3:0] state, state_n;
    // --

    // = Assignments =


/*  assign c0_wb_dat_o = (state==S_CORE_READ) ? m_wb_dat_i : 32'd0;
    assign c0_wb_ack_o = (state==S_CORE_READ) ? m_wb_ack_i : 0;*/
    // --


    always @(*)
    begin

        m_wb_adr_o = 32'd0;
        m_wb_sel_o = 4'b1111;
        m_wb_we_o = 0;
        m_wb_dat_o = 32'd0;
        m_wb_cyc_o = 0;
        m_wb_stb_o = 0;
        m_wb_cti_o = 0;
        m_wb_bte_o = 0;

        ctrl_wb_dat_o = 32'd0;
        ctrl_wb_ack_o = 0;

    `ifdef CORE0
        c0_wb_dat_o = 32'd0;
        c0_wb_ack_o = 0;
    `endif
    `ifdef CORE1
        c1_wb_dat_o = 32'd0;
        c1_wb_ack_o = 0;
    `endif
    `ifdef CORE2
        c2_wb_dat_o = 32'd0;
        c2_wb_ack_o = 0;
    `endif
    `ifdef CORE3
        c3_wb_dat_o = 32'd0;
        c3_wb_ack_o = 0;
    `endif

        state_n = state;

        case (state)
            S_IDLE:
            begin
                if (ctrl_wb_cyc_i) begin
                    m_wb_adr_o = ctrl_wb_adr_i;
                    m_wb_dat_o = ctrl_wb_dat_i;
                    m_wb_we_o  = ctrl_wb_we_i;
                    m_wb_cti_o = ctrl_wb_cti_i;
                    m_wb_bte_o = ctrl_wb_bte_i;
                    state_n = S_CTRL;
                end
            `ifdef CORE0
                else if (c0_wb_cyc_i) begin
                    m_wb_adr_o = c0_wb_adr_i;

                    state_n = S_CORE0;
                end
            `endif
            `ifdef CORE1
                else if (c1_wb_cyc_i) begin
                    m_wb_adr_o = c1_wb_adr_i;

                    state_n = S_CORE1;
                end
            `endif
            `ifdef CORE2
                else if (c2_wb_cyc_i) begin
                    m_wb_adr_o = c2_wb_adr_i;

                    state_n = S_CORE2;
                end
            `endif
            `ifdef CORE3
                else if (c3_wb_cyc_i) begin
                    m_wb_adr_o = c3_wb_adr_i;

                    state_n = S_CORE3;
                end
            `endif

            end
            S_CTRL:
            begin
                m_wb_adr_o = ctrl_wb_adr_i;
                m_wb_sel_o = ctrl_wb_sel_i;
                m_wb_we_o  = ctrl_wb_we_i;
                m_wb_dat_o = ctrl_wb_dat_i;
                m_wb_cyc_o = ctrl_wb_cyc_i;
                m_wb_stb_o = ctrl_wb_stb_i;
                m_wb_cti_o = ctrl_wb_cti_i;
                m_wb_bte_o = ctrl_wb_bte_i;
                ctrl_wb_ack_o = m_wb_ack_i;
                ctrl_wb_dat_o = m_wb_dat_i;
                if (ctrl_wb_cyc_i==0)
                begin
                    state_n = S_IDLE;
                end
            end
        `ifdef CORE0
            S_CORE0:
            begin
                m_wb_adr_o = c0_wb_adr_i;
                m_wb_cyc_o = c0_wb_cyc_i;
                m_wb_stb_o = c0_wb_stb_i;
                c0_wb_ack_o = m_wb_ack_i;
                c0_wb_dat_o = m_wb_dat_i;
                if (c0_wb_cyc_i==0)
                begin
                    state_n = S_IDLE;
                end
            end
        `endif
        `ifdef CORE1
            S_CORE1:
            begin
                m_wb_adr_o = c1_wb_adr_i;
                m_wb_cyc_o = c1_wb_cyc_i;
                m_wb_stb_o = c1_wb_stb_i;
                c1_wb_ack_o = m_wb_ack_i;
                c1_wb_dat_o = m_wb_dat_i;
                if (c1_wb_cyc_i==0)
                begin
                    state_n = S_IDLE;
                end
            end
        `endif
        `ifdef CORE2
            S_CORE2:
            begin
                m_wb_adr_o = c2_wb_adr_i;
                m_wb_cyc_o = c2_wb_cyc_i;
                m_wb_stb_o = c2_wb_stb_i;
                c2_wb_ack_o = m_wb_ack_i;
                c2_wb_dat_o = m_wb_dat_i;
                if (c2_wb_cyc_i==0)
                begin
                    state_n = S_IDLE;
                end
            end
        `endif
        `ifdef CORE3
            S_CORE3:
            begin
                m_wb_adr_o = c3_wb_adr_i;
                m_wb_cyc_o = c3_wb_cyc_i;
                m_wb_stb_o = c3_wb_stb_i;
                c3_wb_ack_o = m_wb_ack_i;
                c3_wb_dat_o = m_wb_dat_i;
                if (c3_wb_cyc_i==0)
                begin
                    state_n = S_IDLE;
                end
            end
        `endif
        endcase
    end


    always @(posedge wb_clk)
    begin
        if (wb_rst) begin
            state <= S_IDLE;
        end
        else begin
            state <= state_n;
        end
    end



endmodule